use crate::config::config::Config;
use regex::Regex;

pub struct CloneArgs {
    pub remote: String,
    pub depth: Option<String>,
}

pub fn exec(args: CloneArgs, config: Config) {
    get_url_props(&args.remote);
}

struct UrlProps<'a> {
    is_secure: bool,
    domain: &'a str,
    path: &'a str,
}

impl UrlProps<'_> {
    fn new() -> Self {
        UrlProps {
            is_secure: true,
            domain: "",
            path: "",
        }
    }
}

pub fn get_url_props(url: &str) -> UrlProps {
    let mut url_props = UrlProps::new();

    // Match protocol and domain
    let re = Regex::new(r"((?<protocol>https?)://)?(?<domain>[^/]*)").unwrap();
    let captures = re.captures(url).expect("fatal: invalid url");

    // Assume is secure
    let protocol = captures.name("protocol").map_or("https", |m| m.as_str());

    url_props.is_secure = protocol == "https";
    url_props.domain = captures
        .name("domain")
        .map(|m| m.as_str())
        .expect("fatal: domain not found");

    // Get rest of url
    let end_of_domain_idx = captures
        .name("domain")
        .expect("Already unwraped before")
        .end();
    let rest_of_url = &url[end_of_domain_idx..];

    // Try webdav url
    if let Some(rest_of_url) = rest_of_url.strip_prefix("/remote.php/dav/files") {
        let re = Regex::new(r"[^\/]*(?<path>.*)").unwrap();
        url_props.path = re
            .captures(rest_of_url)
            .expect("fatal: invalid webdav url")
            .name("path")
            .map_or("/", |m| m.as_str());
        return url_props;
    }

    // Try 'dir' argument
    let re = Regex::new(r"\?dir=(?<path>[^&]*)").unwrap();
    if let Some(captures) = re.captures(rest_of_url) {
        url_props.path = captures.name("path").map_or("/", |m| m.as_str());
        return url_props;
    }

    // Try path next to domain
    if rest_of_url.chars().nth(0).expect("fatal: invalid url") == '/' {
        url_props.path = rest_of_url;
        return url_props;
    }

    panic!("fatal: invalid url (cannot found path)");
}

#[cfg(test)]
mod tests {
    use super::*;
    const DOMAIN: &str = "nextcloud.com";
    const SUBDOMAIN: &str = "nextcloud.example.com";

    fn compare_url_props(url_to_test: &str, is_secure: bool, domain: &str) {
        let path = "/foo/bar";
        let url_props = get_url_props(url_to_test);
        assert_eq!(url_props.is_secure, is_secure);
        assert_eq!(url_props.domain, domain);
        assert_eq!(url_props.path, path);
    }

    #[test]
    fn get_url_props_from_browser_test() {
        compare_url_props(
            "https://nextcloud.com/apps/files/?dir=/foo/bar&fileid=166666",
            true,
            DOMAIN,
        );
        compare_url_props(
            "https://nextcloud.com/apps/files/files/625?dir=/foo/bar",
            true,
            DOMAIN,
        );
    }

    #[test]
    fn get_url_props_direct_url_test() {
        compare_url_props("https://nextcloud.example.com/foo/bar", true, SUBDOMAIN);
        compare_url_props("http://nextcloud.example.com/foo/bar", false, SUBDOMAIN);
        compare_url_props("nextcloud.example.com/foo/bar", true, SUBDOMAIN);
    }

    #[test]
    fn get_url_props_with_port_test() {
        compare_url_props("localhost:8080/foo/bar", true, "localhost:8080");
    }
}
